! $Id: uves_ordpos.prg,v 1.11 2001/09/08 15:28:20 amodigli Exp $
!+++ UVES_ORDPOS ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_ordpos
!
!.KEYWORDS      spectroscopy, echelle, order positions
!
!.USAGE         @p (IDENT) input catalog refer
!
!.INPUT         input   (P1): order position flatfield or standard star
!
!		refer	(P3): reference catalog should contain
!				DRS setup table.
!
!.OUTPUT	catalog (P2): output catalog
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!		Q2: return the classification of the main product
!
!.DEFAULT	P1	ORDREF
!		P2	+
!		P3	+
!
!		DEFMTD=HOUGH DEFPOL=2,3
!
!.PURPOSE       Analyses any echelle exposure in order to determine the order
!		positions. Standard stars, flatfields or arc-lamp exposures
!		as input frames are recommended.
!
!.ALGORITHM     Does a transformation of each echelle frame of the set of
!		of input frame (rotate, split if necessary). Perform a
!		Hough-transform for each exposure using DEFINE/HOUGH which
!		returns the order table as well as the background table.
!
!		The products are classified as:
!		{ORDER_ID(i)}, {BACK_ID(i)} , where i depends on the instrument
!		path. Furthermore, all global keywords belonging to the ECHELLE
!		context are stored in a setup table {DRSRC_ID}.
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/04/23  Created.
!		    1999/06/25	P1 may be an image catalog, too.
!		    1999/07/07	Add some more messages.
!                   1999/07/13	Crossref command implemented.
!                   1999/07/14	Create proper settings for DRS table (SETDRS).
!                   1999/07/19	Set default: P1=ORDREF; add some comments.
!		    1999/09/06	uves_updtbl is obsolete using from now as
!				98NOVpl2.2 has fixed the bug in necbackprep.prg
!                   1999/09/16	Parameters for classify/product 'properly' set.
!                   1999/09/22	FORCE_DRS implemented.
!				DELTA set to 10 (BLUE), 5 (RED).
!                   1999/09/29	DC=4
!                   1999/10/06	WLCITER(2) = 0.1
!                   1999/10/12	Name of output tables now includes cent. wavel.
!				and binning.
!                   1999/10/15	order/width determination removed, not
!				necessary! Caused problems using std.stars!
!                   1999/11/03	Correct SLIT keyword for bin size (CCDBIN(2)).
!                   1999/12/10	return the main product as Q2
!
!.COMMENT	(1) The following global ECHELLE keywords are fixed and may
!		not be changed by the user: DEFMTD, ORDTAB, BAKTAB
!
!		(2) We need uves_updtbl as the background table uses a 
!		descriptor STEP which will be deleted as soon as the table is
!		saved as a FITS file! The procedure uves_updtbl.prg creates a
!		copy of STEP which may be read after the FITS-file version of
!		the background table has been loaded in order to restore STEP !
!		   SO, EVERYWHERE where the background table is in use this
!		procedure should be used. To ensure this, the call will be
!		made in INITIALIZE/PIPELINE and FINISH/PIPELINE -- both
!		routines are obligatory procedures for recipes!
!			*** (2) is obsolete since 98NOVpl2.2 ***
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.11 $
! $Log: uves_ordpos.prg,v $
! Revision 1.11  2001/09/08 15:28:20  amodigli
! PIPEFILE->h_pipefile
! added h_namecros,h_cwlen, h_fits_nameccd
! as asked from PSO
!
! Revision 1.10  2001/08/12 16:14:49  amodigli
! Passing p_version as string par
!
! Revision 1.9  2001/07/15 14:18:57  amodigli
! Eliminated unnecessary call to uves_getpversion.prg
!
! Revision 1.8  2001/07/04 07:55:04  amodigli
! New QC1 naming convention setting
!
! Revision 1.7  2001/06/20 14:21:14  amodigli
! Using QC1 LOG and PAF files
!
! Revision 1.6  2001/06/16 07:03:27  amodigli
! Changing QC Kay naming conventions
!
! Revision 1.5  2001/05/02 14:14:38  amodigli
! New bkg setting now binning dependent
!
! Revision 1.4  2001/02/09 18:10:06  amodigli
! Added PRO QC keywords for DFO
!
! Revision 1.3  2001/01/04 10:39:39  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref input catalog refer
!
! Check parameters
!
define/param p1 ORDREF c "Enter name of flatfield frame or catalog:"
define/param p2 + c "Enter name of OUTPUT catalog:"
define/param p3 + c "Enter name of REFERENCE catalog:"
!
! local constant declaration
!
define/local pid/c/1/16 ordpos
!
! local variable declaration
!
define/local date/c/1/30 " " all +lower_levels
define/local fname/c/1/30 " " all +lower_levels

define/local catal/i/1/1 0
define/local num/i/1/1 0
define/local ifrm/c/1/100 " " ALL
define/local wcen/c/1/10 " " ALL
define/local bin/c/1/4 " " ALL
define/local ext/c/1/4 " " ALL
define/local modext/c/1/25 " " ALL
define/local n_pred/i/1/1 0
define/local n_ord/i/1/1 0

define/local qc_nall/i/1/1 0
define/local qc_nsel/i/1/1 0
define/local qc_ord_min/i/1/1 0
define/local qc_ord_max/i/1/1 0
define/local qc_resid_min/d/1/1 0
define/local qc_resid_max/d/1/1 0
define/local qc_resid_mean/d/1/1 0.0
define/local qc_resid_sigma/d/1/1 0.0
define/local dpr_type/c/1/80 " " all
define/local tpl_id/c/1/80 " " all
define/local arcfile/c/1/80 " " all
define/local pro_catg/c/1/80 " " all


if NICE_CREA .eq. "Y" assign/disp d,1 >Null	!Select display

						!Check if output catalog exist
if m$exist(p2) .eq. 0 .AND. p2(1:1) .ne. "+" -
  create/icat {p2} NULL {UVES_IDSTRNG} >Null


						!Check input type.
parse/path {p1}					!Get file extension.
ext = m$upper(q3)
if ext(1:4) .eq. ".CAT" then			!Check if we got a catalog.
  show/icat {p1} >Null
  num = outputi(1)				!Check number of entries.
  if num .lt. 1 goto ERR1			!No entries in the catalog.
  goto CATALOG
else if ext(1:4) .eq. ".BDF" then
  ifrm = p1
  goto SINGLE
else
  goto ERR2					!Bad P1 specification
endif

CATALOG:
  store/frame ifrm {p1} 1 FINI

SINGLE:
  if m$exist(ifrm) .eq. 0 goto ERR3
  mess/out 3 {pid} "Do the order detection using {ifrm} ..."

  config/instr {ifrm}
  modext = q2					!Mode extension: _REDU,_REDL,..
  outputi = wcenter / 10
  set/format I1
  wcen = "{outputi}"
  set/format F1.0
  if PATHID .lt. 3 then
    bin = "{modext(5:5)}{CCDBIN(1)}x{CCDBIN(2)}"
  else
    bin = "_{CCDBIN(1)}x{CCDBIN(2)}"
  endif
  set/format

  if P3(1:1) .ne. "+" then
    search/icat {p3} {DRSRC_ID({PATHID})} >Null	!Get the DRS setup table
    if m$strlen(out_a) .eq. 0 then		!... does not exist
      append/out "No appropriate DRS setup table found!"
      if FORCE_DRS(1:1) .eq. "Y" then		!... check if it should be used
        append/out "-> skip input frame: `{ifrm}'"
        mess/out 1 {pid}
        goto SKIP
      endif
      mess/out 2 {pid}
      mess/out 3 {pid} "Use global keyword settings (SHOW/ECHELLE)!"
    else					!^ the global keywords of the
      DRSTAB = out_a				!ECHELLE-session.
      savini/echelle {DRSTAB} READ
      mess/out 3 {pid} "Use DRS setup table: `{DRSTAB}`"
      mess/out 4 {pid} "Initialize context Echelle using `{DRSTAB}`"
    endif
  else
    mess/out 2 {pid} "Use global keyword settings (SHOW/ECHELLE)!"
  endif

  set/echelle BAKTAB="b{wcen}{bin}.tbl" 
  set/echelle ORDTAB="o{wcen}{bin}.tbl" ORDREF="{ifrm}"
  outputi =  NBORDI + WIDTHI + THRESI
  if outputi .eq. 0 -
    mess/out 3 {pid} "Do an automatic order detection!"
  
  
!==========================================================
! Start QC Check on predicted vs. detected number of orders
!==========================================================   
n_pred = NBORDI    
n_ord = NBORDI


CHECK:
 
  define/hough P2={n_ord} {SESSOUTV}			!Determine the order positions
  STATISTIC/TABLE {ORDTAB} :X {SESSOUTV} 
  qc_nall = outputi(2)

  STATISTIC/TABLE {ORDTAB} :RESIDUAL {SESSOUTV} 
  qc_nsel = outputi(2)

  STATISTIC/TABLE {ORDTAB} :ORDER {SESSOUTV} 
  qc_ord_min = outputr(1)
  qc_ord_max = outputr(2)

  STATISTIC/TABLE {ORDTAB} :RESIDUAL {SESSOUTV} !Check if order definition if OK
  if outputr(4) .ge. 1.0 then                   !If NOT decrease No of orders
     n_ord = NBORDI -1
     if n_ord .lt. 1 GOTO ERR4
     GOTO CHECK
  endif

  !Remember dimension of :RESIDUAL column in ORDTAB is [pix]
  qc_resid_min = outputr(1)
  qc_resid_max = outputr(2)
  qc_resid_mean = outputr(3)
  qc_resid_sigma = outputr(4)
 

      
						!using the Hough-Transform
  copy/dd {ifrm} *,3 {ORDTAB}			!Copy HIERARCHICAL keywords
  copy/dd {ifrm} *,3 {BAKTAB}			!from input to products
  
						!
  classify/product {ORDTAB} {ORDER_ID({PATHID})} {d_pre} {d_geo} OK
  classify/product {BAKTAB} {BACK_ID({PATHID})} {d_pre} {d_geo} OK

  DRSTAB = "d{wcen}{bin}.tbl"			!name of setup table
  @p uves_ordpos,SETDRS {DRSTAB} {ifrm}		!create DRS setup table
  classify/product {DRSTAB} {DRSRC_ID({PATHID})} {d_pre} {d_empty} OK
  if p2(1:1) .ne. "+" then
    add/icat {p2} {ORDTAB} >Null
    add/icat {p2} {BAKTAB} >Null
    add/icat {p2} {DRSTAB} >Null
  endif

  mess/out 3 {pid} "Order table `{ORDTAB}` created."
  mess/out 3 {pid} "Background table `{BAKTAB}` created."
  mess/out 3 {pid} "DRS setup table `{DRSTAB}` created."
  
!=============================
! output to the logging system
!============================= 


if m$existd(ifrm,h_origname) .eq. 1 then
  fname = "{{ifrm},{h_origname}}"
else
  fname = "{{ifrm},{h_obsname}}_{{p1},{h_obstpno}}"
endif

date   = "{{ifrm},ESO.TPL.START}"

!
! output in ops log and FITS
!
set/format F13.6
!  
!@p uves_qc_log_key 2 {ifrm} {ifrm} P7={date}

 
qc1log/out 2 
set/format i1 f8.4,f8.4
qc1log/out 1 {ORDTAB} "{h_qc_did}"  "{d_qc_did}" 
qc1log/out 1 {ORDTAB} "PRO.REC1.PIPE.ID" "{p_version}" "Pipeline (unique) identifier" PRO
qc1log/out 1 {ORDTAB} "QC.TEST1.ID" "Order-Definition-Results" 

if m$existd(ifrm,h_dprtype) .eq. 1 then
  dpr_type = "{{ifrm},{h_dprtype}}"
  qc1log/out 1 {ORDTAB} {h_dprtype} "{dpr_type}" + DPR
endif
if m$existd(ifrm,h_tpid) .eq. 1 then
  tpl_id = "{{ifrm},{h_tpid}}"
  qc1log/out 1 {ORDTAB} {h_tpid} "{tpl_id}" + TPL
endif
if m$existd(ifrm,h_arcfile) .eq. 1 then
  arcfile = "{{ifrm},{h_arcfile}}"
  qc1log/out 1 {ORDTAB} {h_arcfile} {arcfile} + PRIMARY-FITS
endif
if m$existd(ORDTAB,h_procatg) .eq. 1 then
  pro_catg = "{{ORDTAB},{h_procatg}}"
  qc1log/out 1 {ORDTAB} {h_procatg} {pro_catg} + PRO
endif
qc1log/out 1 {ORDTAB} {h_pipefile} {ORDTAB} + PRO
 
qc1log/out 1 {ORDTAB} "{h_namecros({PATHID})(5:18)}"       {CROSS}  "Cross disperser ID"             UVES_ICS
qc1log/out 1 {ORDTAB} "{h_cwlen({PATHID})(5:)}"     {{ORDTAB},{h_cwlen({PATHID})}}     "Grating central wavelength [nm]" UVES_ICS
qc1log/out 1 {ORDTAB} "{h_fits_nameccd({PATHID})(5:)}"     {CCD}    "Grating unique ID"                CCDDCS

qc1log/out 1 {ORDTAB} "QC.ORD.RESIDMIN"    {qc_resid_min}   "min resid in ord def" 
qc1log/out 1 {ORDTAB} "QC.ORD.RESIDMAX"    {qc_resid_max}   "max resid in ord def" 
qc1log/out 1 {ORDTAB} "QC.ORD.RESIDAVG"    {qc_resid_mean}  "mean resid in ord def" 
qc1log/out 1 {ORDTAB} "QC.ORD.RESIDRMS"    {qc_resid_sigma} "sigma resid in ord def" 
qc1log/out 1 {ORDTAB} "QC.ORD.NPRED"       {n_pred}         "predicted number of orders"    
qc1log/out 1 {ORDTAB} "QC.ORD.NDET"        {n_ord}          "detected number of orders"
qc1log/out 1 {ORDTAB} "QC.ORD.NPOSALL"     {qc_nall}        "Number of position found"
qc1log/out 1 {ORDTAB} "QC.ORD.NPOSSEL"     {qc_nsel}        "Number of position selected"
qc1log/out 1 {ORDTAB} "QC.ORDMIN"          {qc_ord_min}     "minimum order number detected"
qc1log/out 1 {ORDTAB} "QC.ORDMAX"          {qc_ord_max}     "maximum order number detercted"
qc1log/out 3
 
set/format


!==========================================================
! End QC Check on predicted vs. detected number of orders
!==========================================================      







SKIP:
  if ext(1:4) .eq. ".BDF" goto FINI		!single frame -> finished.
  goto CATALOG
FINI:
return 0 {ORDER_ID({PATHID})}

						!Error handling
ERR1:
  mess/out 1 {pid} "Empty input catalog !"
  return -1
ERR2:
  mess/out 1 {pid} "You have to specify full name of input frame (.cat, .bdf)!"
  return -1
ERR3:
  mess/out 1 {pid} "Input file `{ifrm}` does not exist !"
  return -1
ERR4:
  mess/out 1 {pid} "Order Check Failed. Fpund 0 orders!"
  return -1  



!=== SUBROUTINES ==============================================================

ENTRY SETDRS

define/param p1 ? c "Enter name of DRS setup table:"
define/param p2 ? c "Enter name of input frame:"
 

!general information:
set/echelle INSTR=UVES

!order definition:
!The global keywords: DEFMTD, DEFPOL, WIDTHI, THRESI, NBORDI
!may have previously been set, keep them!
!defaults: DEFMTD=HOUGH DEFPOL=2,3 WIDTHI=0, THRESI=0 NBORDI=0

!background subtraction:
!old setting
!set/echelle BKGMTD=SPLINE BKGSTEP=100 BKGRAD=10,2 BKGVISU=YES
!set/echelle BKGDEG=3 BKGSMO=1000.0
 
!am 02/05/01 
!new improved setting
set/echelle BKGMTD=SPLINE BKGVISU=YES
  
define/local sizeX/i/1/1  0
define/local sizeY/i/1/1  0
sizeX = m$value({p2},NPIX(1))
sizeY = m$value({p2},NPIX(2))
 
define/local myBkgStep/i/1/1  0
define/local radX/i/1/1 0
 
!We resize to detector active size taking in count of binning 
!to have the same number of sampling points for any Blue/Red 
!and binning setting
!radX we take half of the step to cover uniformely the detector on
!boxes center at the sampling points and going 
!from -radX+xSample to xSample+radX
!on Y we keep a minimum of 2 pix width
myBkgStep = m$nint(50*{sizeX}/4096)
radX = m$nint(25*{sizeX}/4096)
 
set/echelle BKGSTEP={myBkgStep}
set/echelle BKGRAD={radX},2
set/echelle BKGDEG=1 
!to have high precision.
!next a post smoothing after bakground subtraction will be necessary
set/echelle BKGSMO=0 
 
 
 

!order extraction:
set/echelle EXTMTD=AVERAGE NSKY=2 POSSKY=AUTO OFFSET=0
set/echelle SKYWIND=-22,-7,7,22 SKYOPT=YES	!The sky window will be
SLIT = 15 / CCDBIN(2)				!determined in REDUCE/UVES

!wavelength calibration:
set/echelle SEAMTD=GAUSSIAN WIDTH2=8.0 THRES2=0.0 WLCMTD=GUESS GUESS=wcal
set/echelle WLCREG=STANDARD WLCOPT=2D DC=4 TOL=0.2 WLCNITER=5,30 
set/echelle WLCVISU=YES
WLCITER(2) = 0.1

!resampling:
set/echelle REBOPT=NO REBMTD=NO SAMPLE=+++	!nonlinear resampling

!flux calibration:
set/echelle RESPOPT=NO RESPMTD=STD
set/echelle STD=+++ RESPONSE=response FLUXTAB=unity

!merging:
set/echelle MGOPT=YES MRGMTD=AVERAGE
set/echelle DELTA=10				!BLUE arm
if PATHID .lt. 3 set/echelle DELTA=5		!RED arm


!cosmic ray rejection
set/echelle CROPT=YES EXTSIGMA=5.0

save/drs {p1} {ORDTAB}				!save global keys as descript.
						!in DRS setup table {p1}.
return 0
