/* $Id: physmodtbl.c,v 1.4 2001/05/30 16:33:34 amodigli Exp $ */
/*+++ uves_physmodtbl.c +++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		uves_physmodtbl.exe
 
 .KEYWORDS	uves physical model, spectroscopy, echelle,
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT		IN_A:	Line catalog name,
		IN_B:	line table name,
		FRSPRG:	free spectral range table name,
		ArmId:	Arm index,
		CHIP:	Chip index,
		CDID:	Cross-diperser index,
		wcent:	central wave,
		xtrans, ytrans:	   offset position (x,y),
		echoff, CDoff, ccdrotoff:    offset angle (echelle,CD,CCD-rot),
		pressure, tempcam, humid:  pressure, average camera temperature, 
		slitwidth: slit width.
		
 		 
 .OUTPUT	pline.tbl: predicted positions table, 
 		modfree_sp_rg.tbl: free spectral range table
 
 .RETURN	Q1:  0: successful return
 		    -1: a successful return failed
 
 .PURPOSE	Running the UVES physical model and compute the projection table
 		with predicted positions (x,y) of the ThAr lines. 
 
 .ALGORITHM	The calculations are all based on the physical model routines
                stored in the C library libphysmod.a and done in SI units.

		These input keywords will configure the physical model at the
		beginning.

 .ENVIRON	MIDAS, UVES context
    
 .LANGUAGE	C
 
 .AUTHOR	Olivier BOITQUIN, ESO-DMD
 
 .VERSION	1.0 1999/07/01	Creation
                    1999/09/30-SW  Binning of the physical model implemented.
		                   Terminal output improved.
		    1999/10/06-OB  New columns added in the "modfree_sp_rg.tbl": 
		    computes the pixel-scale taking into account
		    of the anamorphic magnification of CD, length of the line.
		    2000/01/17-OB  New columns added in the "modfree_sp_rg.tbl":
		    Y position of the central column.

 .COMMENT       All calculations of the physical model routines are made
                in nm. So, the input wavelength and results have to be trans-
		formed within this code.
----------------------------------------------------------------------------*/
/* ===========================================================================
 * $Name:  $
 * $Revision: 1.4 $
 * $Log: physmodtbl.c,v $
 * Revision 1.4  2001/05/30 16:33:34  amodigli
 * Taken out error check disabling
 *
 * Revision 1.3  2001/01/04 13:36:30  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */
#ifdef __STDC__  /* ANSI */
  static void  error(void);
#else  /* K&R */
  static void  error();
#endif

#include <tbldef.h>
#include <midas_def.h>
#include <stdlib.h>
#include "physmod.h"
#define     PROID "physmodtbl"
/* for messout function: no "static" (otherwise no messages from other files) */
int messlevel;

#ifdef __STDC__  /* ANSI */
  void main(int argc, char *argv[])
#else  /* K&R */
  void main(argc, argv)
  int  argc;
  char *argv[];
#endif
{

  int tid    = 0, outtid = 0 ;
  int ncol   = 0;
  int nsort  = 0;
  int allcol = 0, ordcol, wavcol, xcol,ycol;
  int allrow = 0;
  int actvals, kunit;
  int aRowNumber, col, null, i, imod = 0, mm;

/* for the computation of the physical model */  

  char    linetbl[200], modtbl[200], freemodtbl[200], aHeading[200], line[200];
  char    ArmId[2], CHIP[2], reg_opt[2];
  double  dblwav, wcent, xmod,ymod, xreg,yreg, xtrans,ytrans, echoff,CDoff,ccdrotoff;
  double  pressure, tempcam, humid;
  int     CDID, cfgindx, xpix, ypix, order;   
  float   CCDBIN[2];

/* for the computation of the free spectral range */

  double x,y, x1, y1, l, dl, ls, le, fsr_min, fsr_max,xfsr_min,yfsr_min,xfsr_max,yfsr_max;
  double xr,yr,lcent;
  int    m, m_min, m_max;
  double lmin,lmax, dxpix, dypix;
  int    mmin,mmax;
  int 	 startcol,endcol,ycentcol,ycent,fsrmincol,fsrmaxcol,xfsrmincol,xsrmaxcol,yfsrmincol,ysrmaxcol,bincol;

/* for messout */
  char  tmps[200];
  
/* For the computation of the width (in pixel and A) and resolution lines */  
  double   slitwidth, binsize, linewidth, linewidpx, resol; 
  int      binScol,lineWcol,lineWPXcol,lineLPXcol,resolcol,centcol,pixsclcol,pixsclCDcol;
  
/* For the computation of the true pixel-scale along echelle dispersion */
  double   scale, pixscale, pixscalCD, linelenpx, slitlength ;

/* For the computation of the anamorphic magnification and the blaze function */
  double   blaze, betacd, betaech, fc, rech, rcd;
  int      blzcol;

/*  int binx, biny; */

/* As usual */
  int   r, nval, unit;
  
  SCSPRO("uves_phymod");
  SCKRDI("MESS_LEVEL",1,1,&nval,&messlevel,&unit,&null);  
  
  SCKGETC("IN_A", 1, 60, &actvals, linetbl);
  SCKGETC("OUT_A", 1, 60, &actvals, modtbl);
  SCKGETC("FRSPRG", 1, 60, &actvals, freemodtbl);
    
/* Get the configuration keywords */

  SCKRDC("ArmId",1,1,1,&actvals,ArmId,&kunit,&null);
  SCKRDC("CHIP",1,1,1,&actvals,CHIP,&kunit,&null);
  SCKRDR("CCDBIN",1,2,&actvals,CCDBIN,&kunit,&null);
  SCKRDI("CDID",1,1,&actvals,&CDID,&kunit,&null);
  SCKRDD("wcent",1,1,&actvals,&wcent,&kunit,&null);
  SCKRDD("xtrans",1,1,&actvals,&xtrans,&kunit,&null);
  SCKRDD("ytrans",1,1,&actvals,&ytrans,&kunit,&null);  
  SCKRDD("echoff",1,1,&actvals,&echoff,&kunit,&null);
  SCKRDD("CDoff",1,1,&actvals,&CDoff,&kunit,&null);
  SCKRDD("ccdrotoff",1,1,&actvals,&ccdrotoff,&kunit,&null);
  SCKRDD("pressure",1,1,&actvals,&pressure,&kunit,&null);
  SCKRDD("tempcam",1,1,&actvals,&tempcam,&kunit,&null);
  SCKRDD("humid",1,1,&actvals,&humid,&kunit,&null);
  SCKRDD("slitwidth",1,1,&actvals,&slitwidth,&kunit,&null);
  SCKRDD("slitlength",1,1,&actvals,&slitlength,&kunit,&null);
  SCKRDC("reg_opt",1,1,1,&actvals,reg_opt,&kunit,&null);
    
  TCTOPN(linetbl, F_I_MODE, &tid);

  TCIGET (tid, &ncol, &aRowNumber, &nsort, &allcol, &allrow); 
  
  sprintf(&tmps[0],"Opened table %s ncol=%d nrow=%d",linetbl ,ncol,aRowNumber);
  messout(messlevel,3,PROID,tmps);  

  strcpy(aHeading,":WAVE");
  TCCSER(tid, aHeading, &col);
  
  if (col <= 0) {
    sprintf(&tmps[0],"Could not find column: %s",aHeading);
    messout(messlevel,1,PROID,tmps);
    error();
  }
  
/* Initialization of configuration */

  airConfig(pressure/1.33322, tempcam, humid);
  /*printf("Air Index = %f\n",airIndex(wcent)); */
  /*printf("Shift in x = %f, shift in y = %f\n",xtrans,ytrans); */
  /*printf("Offset of CCD rotation %f\n",ccdrotoff); */
  sprintf(&tmps[0],"Air Index = %f\nShift in x = %f, shift in y = %f",airIndex(wcent),xtrans,ytrans);
  messout(messlevel,3,PROID,tmps);
  sprintf(&tmps[0],"Offset angle of echelle, CD, CCD rotation :\n%f, %f, %f",echoff,CDoff,ccdrotoff);
  messout(messlevel,3,PROID,tmps);

  setccdrot(ccdrotoff, ccdrotoff);
  setIncidence(echoff, echoff, CDoff, CDoff);
/*  setBinning(binx, biny); */
  cfgindx = uvesConfig(ArmId[0],CHIP[0],CDID,wcent,CCDBIN[0],CCDBIN[1]);
  
/* Computation of the true pixel-scale along echelle dispersion */
/* scale = (K*FocalRatioNasmyth*PixelSize*ApertureCamera*1e-3)/FocalNasmyth
   K = 206265 : conversion radian to arcsec (1 radian = 206265 arcsec)*/
/*  scale = (206265.0*15.0*0.015*200*1e-3*CCDBIN[0])/120000; */

/* Creation of modtbl */

  TCTINI(modtbl, F_TRANS, F_IO_MODE, 10, 3*aRowNumber, &outtid);
  
  sprintf(&tmps[0],"table created: %s ncol=%d, nrow=%d",modtbl,10,3*aRowNumber);  
  messout(messlevel,3,PROID,tmps);
  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "WAVE", &wavcol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "I6"   , " ", "ORDER", &ordcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "XMOD", &xcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "YMOD", &ycol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "BIN_SIZE", &binScol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINEWIDPX", &lineWPXcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINEWIDTH", &lineWcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "RESOL", &resolcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINELENPX", &lineLPXcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "BLAZE", &blzcol);
 
  
  ccdSize(&xpix,&ypix); 
  /*
  sprintf(&tmps[0],"Arm = %s, CCD = %s, (xpix=%d,ypix=%d),\nCD=%d, Wcent= %f", ArmId, CHIP, xpix,ypix, CDID, wcent);
  messout(messlevel,3,PROID,tmps);
  */

  for (i=0; i<aRowNumber; i++) {
    TCERDD(tid, (i+1), col, &dblwav, &null);
/*    printf("Line: %f\n",dblwav);  */
    dblwav /= 10.0;     /* (10.0: conversion factor from nm to A)*/
    order = findM(dblwav);
    order++;  

/* Computes the positions xmod and ymod for the order-1, order, order+1 */
/* and adds a row to the modtbl only if the position are insight the CCD */
 
    for (mm=0; mm<3; mm++){
      /*xyModel(dblwav, order, &xmod, &ymod);
      blaze = blz; */
      lm2beta(dblwav, order, &betaech, &betacd, &fc);
      beta2xy(betacd, betaech, fc, &xmod, &ymod);

/* Computes the anamorphic magnification and the blaze function */      
      photobeta(dblwav, betaech, betacd, &rech, &rcd, &blaze); 
     
/* Translation */
      xmod += xtrans;
      ymod += ytrans;
       
      if (xmod > 0 && xmod < xpix && ymod > 0 && ymod < ypix) {     

	
/* Computes the width (in pixel and A) and resolution lines */
	  pixscl(dblwav, order, rech, rcd, CCDBIN[0], CCDBIN[1], fc, slitwidth, slitlength, 
			 &binsize, &pixscale, &pixscalCD, 
			 &linewidpx, &linelenpx, &linewidth, &resol);
	       
/* Compute/regress */
	  if (reg_opt[0] == 'Y') {
		xyRegres(xmod,ymod,&xreg,&yreg);
		}
	  else {
	  xreg = xmod;
	  yreg = ymod;
	  }

	  TCEWRD(outtid, ++imod, wavcol, &dblwav);
	  TCEWRI(outtid, imod, ordcol, &order);
	  TCEWRD(outtid, imod, xcol, &xreg);
	  TCEWRD(outtid, imod, ycol, &yreg);
	  TCEWRD(outtid, imod, binScol, &binsize);
	  TCEWRD(outtid, imod, lineWPXcol, &linewidpx);
	  TCEWRD(outtid, imod, lineWcol, &linewidth);
	  TCEWRD(outtid, imod, resolcol, &resol);
	  TCEWRD(outtid, imod, lineLPXcol, &linelenpx);
	  TCEWRD(outtid, imod, blzcol, &blaze);
      }
      order--;
    }
  }

/* close everything */

  TCTCLO(tid); 
  TCTCLO(outtid);


  sprintf(&tmps[0],"Predicted number of lines: %d",imod);
  messout(messlevel,3,PROID,tmps);
/* Compute the free spectral range */

 sprintf (&tmps[0],"Determine the Free Spectral Range\n");
 messout(messlevel,3,PROID,tmps);
 
   dxpix = (double) xpix;
   dypix = (double) ypix;

   m_min = -1;
   m_max = -1;
/*
   lmModel(&l,&m_max,dxpix/2,1.0);
   lmModel(&l,&m_min,dxpix/2,dypix);
*/
  lmModel(&lmax,&m_min,1.0,dypix);
  lmModel(&lmin,&m_max,dxpix,1.0); 

  lmModel(&lmax,&m_min,dxpix,dypix);
  lmModel(&lmin,&m_max,1.0,1.0); 
   
/*   printf("m_min= %d,m_max= %d\n",m_min,m_max); */

   
/* Creation of the modfree_sp_rg.tbl table */

  TCTINI(freemodtbl, F_TRANS, F_IO_MODE, 20,m_max-m_min+1, &outtid);
  
  sprintf(&tmps[0],"FSR-table created: %s ncol=%d, nrow=%d",freemodtbl,20,m_max-m_min+1);  
  messout(messlevel,3,PROID,tmps);
  
  TCCINI(outtid, D_R8_FORMAT, 1, "I6"   , " ", "ORDER", &ordcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "WVCENTCOL", &centcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "YCENTCOL", &ycentcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "START", &startcol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "END", &endcol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "WAVECENT", &wavcol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "YCENT", &ycent);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "FSRMIN", &fsrmincol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "FSRMAX", &fsrmaxcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "XFSR_MIN", &xfsrmincol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "XFSR_MAX", &xsrmaxcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "YFSR_MIN", &yfsrmincol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "YFSR_MAX", &ysrmaxcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "PIXSCALE", &pixsclcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "PIXSCALCD", &pixsclCDcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "BIN_SIZE", &bincol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINEWIDPX", &lineWPXcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINEWIDTH", &lineWcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "RESOL", &resolcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINELENPX", &lineLPXcol);

  imod = 0;
/*
  printf("m, waveCentcol,   Start,     End,       yCent,    FSRmin,    FSRmax,  bin (mA)\n");
*/
  for (m=m_max; m>=m_min; m--) {
     /*printf("%d ",m);*/
     findFSR(m,&l, &fsr_min, &fsr_max);
     
     xyModel(l,m,&x,&y);

/* Translation */
      x += xtrans;
      y += ytrans;
     
     xyModel(fsr_min,m,&xfsr_min,&yfsr_min);
     
/* Translation */
      xfsr_min += xtrans;
      yfsr_min += ytrans;

     xyModel(fsr_max,m,&xfsr_max,&yfsr_max);
     
/* Translation */
      xfsr_max += xtrans;
      yfsr_max += ytrans;

/* Computes the anamorphic magnification and the blaze function */      
     lm2beta(l,m, &betaech, &betacd, &fc);
     photobeta(l, betaech, betacd, &rech, &rcd, &blaze); 
     
/* Computes the width (in pixel and A) and resolution lines */  
     pixscl(l, m, rech, rcd, CCDBIN[0], CCDBIN[1], fc, slitwidth, slitlength, 
            &dl, &pixscale, &pixscalCD, 
	    &linewidpx, &linelenpx, &linewidth, &resol);

     mmin  = m;
     mmax = m;

/* WARNING! The central column doesn't correspond to the central wavelength position */     
     lmModel(&lcent,&m,dxpix/2,y);
     xyModel(lcent,m,&xr,&yr);
     
     lmModel(&lmax,&mmax,dxpix,y);
     lmModel(&lmin,&mmin,1.0,y); 

/*     if (y>=0. && y <=dypix) {*/
       sprintf(&tmps[0],"m, waveCent (nm), Start, End, yCent, FSRmin, FSRmax, bin (mA) = %d %f  %f %f %f %f %f %f\n",m,l,lmin,lmax,y,fsr_min,fsr_max,dl);
/*
       printf("%d %f %f %f %f %f %f %f\n",m,lcent,lmin,lmax,yr,fsr_min,fsr_max,dl);
*/
/*     SCTPUT(tmps); */
       messout(messlevel,4,PROID,tmps);
       sprintf(&tmps[0],"pixel-scale = %f, anamorphic corrections = %f, %f\n", pixscale, rech, rcd);
       messout(messlevel,4,PROID,tmps);
       
       TCEWRI(outtid, ++imod, ordcol, &m);
       TCEWRD(outtid, imod, centcol, &lcent);
       TCEWRD(outtid, imod, ycentcol, &yr);
       TCEWRD(outtid, imod, startcol, &lmin);
       TCEWRD(outtid, imod, endcol, &lmax);
       TCEWRD(outtid, imod, wavcol, &l);
       TCEWRD(outtid, imod, ycent, &y);
       TCEWRD(outtid, imod, fsrmincol, &fsr_min);
       TCEWRD(outtid, imod, fsrmaxcol, &fsr_max);
       TCEWRD(outtid, imod, xfsrmincol, &xfsr_min);
       TCEWRD(outtid, imod, xsrmaxcol, &xfsr_max);
       TCEWRD(outtid, imod, yfsrmincol, &yfsr_min);
       TCEWRD(outtid, imod, ysrmaxcol, &yfsr_max);
       TCEWRD(outtid, imod, pixsclcol, &pixscale);
       TCEWRD(outtid, imod, pixsclCDcol, &pixscalCD);
       TCEWRD(outtid, imod, bincol, &dl);
       TCEWRD(outtid, imod, lineWPXcol, &linewidpx);
       TCEWRD(outtid, imod, lineWcol, &linewidth);
       TCEWRD(outtid, imod, resolcol, &resol);
       TCEWRD(outtid, imod, lineLPXcol, &linelenpx);
/*     }*/
  }

  SCKWRC("Q1",1,"0",1,60,&unit);
  SCSEPI();  
}

/*---------------------------------------------------------------------*/

#ifdef __STDC__  /* ANSI */
  static void error(void)
#else  /* K&R */
  static void error()
#endif
{
  int  r, unit;

  r = SCKWRC("Q1",1,"1",1,60,&unit);
  r = SCSEPI();
}

/*---------------------------------------------------------------------*/

